/*
 * M5Stamp PICO IoT実験
 * 天気予報を取得～気象庁版～
 */
#include <WiFi.h>
#include <SPI.h>
#include <Wire.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>

#define DEBUG true
#define BAUDRATE 9600
char str[120] ;

#include "ssd1331.h"
#include "sd1307.h"
#include "JMA.h"

// --- OLED ---
SSD1331 ssd1331 ;

// --- RTC 000
SD1307 sd1307 ;

// --- DHT-11
#define DHT_PIN 0
struct dht11_s {
  int8_t humidityN ;   // 湿度の整数部
  int8_t humidityD ;   // 湿度の小数部 ※は常に0
  int8_t temperatureN; // 温度の整数部
  int8_t temperatureD; // 温度の小数部
} dht11 ;


// --- OpenWatheMap ---
JMAWATHER jmaWather ;

// ----- NetWork Data ---
// ▼ ssid/passwordはご自宅のWi-FiのSSIDとPASSWORDを設定してください。
//    JMA.cpp 内のJMA_URL/AreaCodeも合わせて変更してください
const char* ssid     = "****";
const char* password = "****";

// --- 時刻調整用
#define SPANTIME 60 // データ取得間隔
unsigned long passTime;
bool sendTime = false ;
struct tm timeInfo;

// --------------
// ----- NTP ----
// --------------
void connectWiFi( ) {
  if (WiFi.status() != WL_CONNECTED) {
    WiFi.reconnect( ) ;
    int i = 0 ;
    while (WiFi.status() != WL_CONNECTED) {
      delay(500);
#if DEBUG
      Serial.print(".");
#endif
      i++ ;
      if (i > 20) {
        delay(30000);
        return  ;
      }
    }
#if DEBUG      
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());
#endif
  }
}

void getNTP( ) {
  connectWiFi() ;
  configTime(9 * 3600L, 0, "ntp.nict.jp", "time.google.com", "ntp.jst.mfeed.ad.jp");
  getLocalTime(&timeInfo) ;
  
  sd1307.yer = timeInfo.tm_year ;       // 年
  sd1307.mon = timeInfo.tm_mon + 1 ;    // 月
  sd1307.day = timeInfo.tm_mday ;       // 日
  sd1307.sec = timeInfo.tm_sec;         // 秒
  sd1307.mnt = timeInfo.tm_min ;        // 分
  sd1307.how = timeInfo.tm_hour ;       // 時
  sd1307.weekday = timeInfo.tm_wday + 1; // 曜日(1～7)
  sd1307.setTime( ) ;

#if DEBUG
  sprintf(str,"NEW %04d/%02d/%02d(%d) %02d:%02d:%02d",
        timeInfo.tm_year + 1900 , timeInfo.tm_mon + 1 ,  timeInfo.tm_mday, timeInfo.tm_wday + 1 ,
        timeInfo.tm_hour , timeInfo.tm_min , timeInfo.tm_sec) ;
  Serial.println(str) ;
  sd1307.getTime( ) ;
  sprintf(str,"RTC 20%02d/%02d/%02d(%d) %02d:%02d:%02d",
        sd1307.yer , sd1307.mon ,  sd1307.day, sd1307.weekday ,
        sd1307.how , sd1307.mnt , sd1307.sec) ;
  Serial.println(str) ;

#endif
}

// ----------------
// ----- DHT11 ----
// ----------------
bool readDHT11( ) {
  uint8_t readData[5] ;
  uint8_t checkSum = 0 ;
  int setPos = 0 ;
  int bitLen = 0 ;
  int stat = HIGH ;
  uint8_t bitData ;
  unsigned long beginTime ;
  unsigned long passTime ;
  unsigned long zeroTime = 35 ; // 35μ秒以下なら０ 以上なら１と判定 ※仕様では0は26～28μ秒
  bool flagStart = false;

  for (int i=0;i<5;i++) {
    readData[i] = 0 ;
  }

  // ----- データ送信要求 -----
  pinMode(DHT_PIN, OUTPUT); //開始信号
  delay(20);
  pinMode(DHT_PIN, INPUT);
  while(digitalRead(DHT_PIN) == LOW) {
    delayMicroseconds(5) ;
  }

  // ----- ACSEPT -----
  flagStart = false;
  passTime = 0 ;
  beginTime = micros();
  while (passTime < 1000) {
    passTime = micros() - beginTime ;
    if (digitalRead(DHT_PIN) !=  stat) {
      if (stat == HIGH) {
        stat = LOW ;
        if (flagStart) {
           break ; 
        }
      } else {
        beginTime = micros();
        flagStart = true ;
        stat = HIGH ;
      }
    }
  }
  
  // ----- データ受信 -----
  bitData = 0x80 ;
  flagStart = false;
  passTime = 0 ;
  beginTime = micros();
  while (bitLen < 40 && passTime < 1000) {
    passTime = micros() - beginTime ;
    if (digitalRead(DHT_PIN) !=  stat) {
      if (stat == HIGH) {
        stat = LOW ;
        if (flagStart) {
          // ----- 1Bit 受信 -----
          if (passTime > zeroTime) {
            readData[setPos] |= bitData ;
          }
          if (bitData != 0x01) {
            bitData >>= 1 ;
          } else {
            bitData = 0x80 ;
            setPos ++ ;
          }

          // ----- ステート変数リセット -----
          beginTime = micros();
          flagStart = false;
          bitLen ++ ;
        }
      } else {
        beginTime = micros();
        flagStart = true ;
        stat = HIGH ;
      }
    }
  }

  // ----- 取得データ格納 -----
  dht11.humidityN = readData[0] ;   // 湿度の整数部 Range:5~95
  dht11.humidityD = readData[1] ;   // 湿度の小数部 ※は常に0
  dht11.temperatureN = readData[2]; // 温度の整数部 -20 ～ 
  dht11.temperatureD = readData[3]; // 温度の小数部
  checkSum = readData[0] + readData[1] + readData[2] + readData[3] ;

  return (bitLen == 40 && checkSum == readData[4]) ;
}

// ---------------------
// ----- 気象庁情報 -----
// ---------------------
void DispWaterIcon( ) {
  jmaWather.getWatherData( ) ;
  
   ssd1331.dispWatherIcon(jmaWather.watheData[0].watherCode, 0,32) ;
   ssd1331.dispWatherIcon(jmaWather.watheData[1].watherCode,48,32) ;
}

// -----------------
// ----- 温湿度 -----
// -----------------
void DispTemp( ) {
  if (readDHT11( )) {
    sprintf(str,"%2d.%dC %2d%%", dht11.temperatureN ,dht11.temperatureD,dht11.humidityN ) ;
  } else {
    sprintf(str,"--.--C --%%") ;
  }
  ssd1331.dispStr(str,4,16) ;
}

// ----------------------
// ----- セットアップ -----
// ----------------------
void setup() {
  // --- 初期化 ---
#if DEBUG
  Serial.begin(BAUDRATE);
  Serial.println("\nWakeUp") ;
#endif
  Wire.begin() ;
  SPI.begin();//SPIを初期化、SCK、MOSI、SSの各ピンの動作は出力、SCK、MOSIはLOW、SSはHIGH
  ssd1331.init( ) ;

  // --- Netwaorkに接続
  Serial.println("Networkに接続") ;
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
#if DEBUG
    Serial.print(".");
#endif
  }

#if DEBUG
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
#endif

  // ----- 起動時 初期読み取り -----
  getNTP( ) ;
  DispTemp( ) ;
  DispWaterIcon( ) ;
  sendTime = false ;
}

// ----------------------
// ----- ループ-----
// ----------------------
void loop() {
  sd1307.getTime( ) ;
  sprintf(str,"%02d/%02d %02d:%02d", sd1307.mon ,  sd1307.day , sd1307.how , sd1307.mnt ) ;
  ssd1331.dispStr(str,4,0) ;

  if (sd1307.mnt == 0) {
    if (!sendTime) {
      sendTime = true ;
      if (sd1307.how == 0) {
          getNTP( ) ;
      }

      if (readDHT11( )) {
        sprintf(str,"%2d.%dC %2d%%", dht11.temperatureN ,dht11.temperatureD,dht11.humidityN ) ;
      } else {
        sprintf(str,"--.--C --%%") ;
      }
      ssd1331.dispStr(str,4,16) ;
      DispWaterIcon( ) ;
    }
  } else {
      sendTime = false ;
  }

  // ループ間隔 10秒
  delay(1000) ;
}
