// --- OpenWatherMap用 ---
#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include "JMA.h"

// ▼ JMA-UELとAreaCodeはお住まいの地域に合わせて書き換えてください
//    140000 は神奈川県　140020は神奈川西部のAreaCodeです。
const String JMA_URL = "https://www.jma.go.jp/bosai/forecast/data/forecast/140000.json" ;
const String AreaCode = "140020" ;

void JMAWATHER::getWatherData( ) {
  HTTPClient http;
  http.begin(JMA_URL); // URLを指定
  int httpCode = http.GET();  // GETリクエストを送信
  if (httpCode == 200) { // Request Result 正常終了
    String watherData = http.getString();  //返答（JSON形式）を取得
    readWatherData(watherData,AreaCode) ;
#if DEBUG
  } else {
    Serial.print("http Rewquest failed. -- ");
    Serial.println(httpCode);
#endif
  }
}

void JMAWATHER::readWatherData(String watherDataStr,String areaCode) {
  memset(watheData,0,sizeof(watheData)) ;
#if DEBUG
  Serial.println("--- JSON DECODE ---");
#endif
  DynamicJsonDocument doc(1024*6);
  deserializeJson(doc, watherDataStr);

  int areaP = -1 ;
  for (int i = 0 ; i< 10  ; i++) {
    String areaName = doc[0]["timeSeries"][0]["areas"][i]["area"]["name"];
    if ( areaName == NULL) break ;
    String code = doc[0]["timeSeries"][0]["areas"][i]["area"]["code"];
#if DEBUG
    Serial.print("area name =");
    Serial.print(areaName);
    Serial.print("  area code =");
    Serial.println(code);
#endif
    if (code.compareTo(areaCode) == 0) {
      areaP = i  ;
      break ;
    }
  }
  // --
  if (areaP >= 0) {
    for (int i = 0 ; i< 2 ; i++) {
      watheData[i].tim = dateFormat(doc[0]["timeSeries"][0]["timeDefines"][i]) ;
      String watherStr = doc[0]["timeSeries"][0]["areas"][areaP]["weathers"][i];
      watheData[i].wather = watherStr ;
      watheData[i].watherCode = doc[0]["timeSeries"][0]["areas"][areaP]["weatherCodes"][i];
#if DEBUG
      Serial.println(" ") ;
      Serial.println(watheData[i].wather) ;
      Serial.print("watherCode = ") ;
      Serial.println(watheData[i].watherCode) ;
      Serial.print(watheData[i].tim.tm_year) ;
      Serial.print("/") ;
      Serial.print(watheData[i].tim.tm_mon) ;
      Serial.print("/") ;
      Serial.print(watheData[i].tim.tm_mday) ;
      Serial.print(" ") ;
      Serial.print(watheData[i].tim.tm_hour) ;
      Serial.print(":") ;
      Serial.println(watheData[i].tim.tm_min) ;
#endif
    }
  }
}

WatherTime_t JMAWATHER::dateFormat(String dateStr) {
  WatherTime_t tim ;
  int dateData[6] ;
  String dateDisp = "" ;
  int l = dateStr.length( ) ;
  int p = 0 ;
  for (int i = 0; i < 6 ; i++) {
    dateData[i] = 0 ;
  }
  for (int j=0;j<l;j++) {
    if (dateStr.charAt(j) == '+') break ;
    if ('0' <= dateStr.charAt(j) && dateStr.charAt(j) <= '9') {
      dateData[p] = dateData[p] * 10 + (dateStr.charAt(j) - '0') ;
    } else {
      p ++ ;
    }
  }
  tim.tm_year = dateData[0] ;  // 年
  tim.tm_mon  = dateData[1];   // 月 - 1 (0-11)
  tim.tm_mday = dateData[2];   // 日 (1-31)
  tim.tm_hour = dateData[3];   // 時 (0-23)
  tim.tm_min  = dateData[4];   // 分 (0-59)
  tim.tm_sec  = dateData[5];   // 秒 (0-59)
  
  return tim ;
}
