#include <Wire.h>
#include "SD1307.h"

#define RTC_ADDRESS 0x68

// 2進化10進数(BCD)を10進数に変換
byte BCDtoDec(byte value) {
  return ((value >> 4) * 10) + (value & 0x0F) ;
}

void SD1307::setTime( ) {
  Wire.beginTransmission(RTC_ADDRESS);    // I2Cスレーブに対して送信処理を開始する
 
  Wire.write(0x00);  // レジスタのアドレスを指定する(0-63の内、先頭から)
  
  // 2進化10進数(BCD)で指定します。(03以外)
  how = (how/10) * 16 + (how % 10) ;
  mnt = (mnt/10) * 16 + (mnt % 10) ;
  sec = (sec/10) * 16 + (sec % 10) ;
  day = (day/10) * 16 + (day % 10) ;
  mon = (mon/10) * 16 + (mon % 10) ;
  yer = (1900 + yer) % 100 ;
  yer  = (yer/10) * 16 + (yer % 10) ;

  Wire.write(sec & 0x7F);  // [00]Seconds(00秒)
  Wire.write(mnt);  // [01]Minutes(m分)
  Wire.write(how);  // [02]Hours(h時)
  Wire.write(weekday);  // [03]WDay 1:日 2:月 3:火 4:水 5:木 6:金 7:土
  Wire.write(day); // [04]Date
  Wire.write(mon); // [05]Month
  Wire.write(yer); // [06]Years(+1900)
 
  // I2Cスレーブへの送信完了
  Wire.endTransmission();
}

void SD1307::getTime( ) {
  // レジスタのアドレスを先頭にする
  Wire.beginTransmission(RTC_ADDRESS);
  Wire.write(0x00);
  Wire.endTransmission();

  // I2Cスレーブに8byteのレジスタデータを要求する
  if (7 != Wire.requestFrom(RTC_ADDRESS, 7)) return ;
 
  // 8byteのデータを取得する
  int i;
  bool flagDisplay = false ;
  for (i = 0; i < 7; i++) {
    while (Wire.available() == 0 ) {}
    RegTbl[i] = Wire.read();
    if (RegTblPr[i] != RegTbl[i]) flagDisplay = true ;
    RegTblPr[i] = RegTbl[i];
  }
 
  // 現在日時 : グローバル変数にセット
  yer = BCDtoDec(RegTbl[6]) ;
  mon = BCDtoDec(RegTbl[5]) ;
  day = BCDtoDec(RegTbl[4]) ;
  weekday = BCDtoDec(RegTbl[3] & 0x07) ;
  how = BCDtoDec(RegTbl[2]) ;
  mnt = BCDtoDec(RegTbl[1] & 0x7F) ;
  sec = BCDtoDec(RegTbl[0] & 0x7F) ;
}
